# Copyright 2021- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)
from odoo import http
from odoo.http import request
from odoo.osv import expression

from odoo.addons.account.controllers.portal import PortalAccount as PortalAccount
from odoo.addons.http_routing.models.ir_http import slug


class PortalAccount(PortalAccount):
    def _prepare_invoice_surplus_portal_values(self, operation, order):
        """
        Get all surplus invoices matching domain
        """
        values = {}
        values.update(self._get_role(operation))
        domain = expression.AND(
            [
                [("acc_operation_id", "=", operation.id)],
                [("acc_delivery_counter_id", "=", False)],
                [
                    (
                        "move_type",
                        "in",
                        (
                            "out_invoice",
                            "out_refund",
                            "in_invoice",
                            "in_refund",
                            "out_receipt",
                            "in_receipt",
                        ),
                    )
                ],
            ]
        )
        # Si pas PMO, admin ou SuperAdmin, il faut limiter plus les factures retournées
        if (
            not values.get("isSuperAdmin")
            and not values.get("isAdmin")
            and not values.get("isPmo")
        ):
            subdomain = expression.OR(
                [
                    [
                        (
                            "company_id.partner_id",
                            "=",
                            request.env.user.commercial_partner_id.id,
                        )
                    ],
                    [
                        (
                            "acc_operation_id.pmo_id.commercial_partner_id",
                            "=",
                            request.env.user.commercial_partner_id.id,
                        )
                    ],
                ]
            )
            domain = expression.AND([domain, subdomain])
        values["invoices_surplus"] = (
            request.env["account.move"].sudo().search(domain, order=order)
        )
        return values

    @http.route(
        [
            '/operation/<model("acc.operation"):operation>/invoices-surplus',
        ],
        type="http",
        auth="user",
        website=True,
    )
    def invoice_surplus_list(
        self, operation, sortby=None, filterby=None, search=None, search_in="all"
    ):
        """
        List all surplus invoices
        """

        render_values = {}
        render_values.update(self._get_role(operation))

        order = "state, name desc, start_date desc, partner_id"

        render_values["invoices_surplus"] = self._prepare_invoice_surplus_portal_values(
            operation, order
        )["invoices_surplus"]

        render_values.update(
            {
                "default_url": "/operation/" + slug(operation) + "/invoices-surplus",
                "page_name": "invoice-surplus",
                "page_breadcrumbs": "Factures Surplus",
            }
        )

        return request.render(
            "oacc_portal_account_surplus.operation_invoice_surplus", render_values
        )
